import { AsyncEmitter, SyncEmitter } from "../event";
export type AcceptType = 'json' | 'text' | 'blob' | 'arraybuffer' | '';
export interface FetchInitOption extends RequestInit {
    label: string;
    path: string;
    url: string;
    contentType: string;
    acceptType: AcceptType;
    input: any;
    headers: Record<string, string>;
}
export type FetchInitType = Partial<FetchInitOption> | string;
export interface FetchPageInfo {
    pageNumber: number;
    pageSize: number;
    total: number;
}
export interface FetchResult<T = any> {
    data: T;
    pageInfo: FetchPageInfo;
    error?: any;
    response?: Response;
    request: FetchInitOption;
    hasData?: boolean;
}
export type FetchMethod<Response, Request> = (input: Request) => Promise<FetchResult<Response>>;
export interface FetchOption {
    baseUrl: string;
    contentType: string;
    acceptType: AcceptType;
}
export interface Fetcher {
    request<Response, Request>(method: 'GET' | 'POST' | 'DELETE' | 'PUT' | 'HEAD' | 'PATCH', init: FetchInitType): FetchMethod<Response, Request>;
    get<Response = any, Request = any>(init: FetchInitType): FetchMethod<Response, Request>;
    post<Response = any, Request = any>(init: FetchInitType): FetchMethod<Response, Request>;
    head<Response = any, Request = any>(init: FetchInitType): FetchMethod<Response, Request>;
    delete<Response = any, Request = any>(init: FetchInitType): FetchMethod<Response, Request>;
    patch<Response = any, Request = any>(init: FetchInitType): FetchMethod<Response, Request>;
    put<Response = any, Request = any>(init: FetchInitType): FetchMethod<Response, Request>;
    onRequest: (ctx: AsyncEmitter<FetchResult<any>>) => void;
    onResponse: (ctx: AsyncEmitter<FetchResult<any>>) => void;
    onFinish: (ctx: SyncEmitter<FetchResult<any>>) => void;
    use: <T>(func: (fetcher: this, options?: T) => void, options?: T) => void;
    setFetchOption: (option: Partial<FetchOption>) => void;
    getFetchOption: () => FetchOption;
}
export declare class MinFetch implements Fetcher {
    private em;
    private emitRequest;
    onRequest: (v: AsyncEmitter<FetchResult<any>>) => void;
    private emitResponse;
    onResponse: (v: AsyncEmitter<FetchResult<any>>) => void;
    private emitFinish;
    onFinish: (v: SyncEmitter<FetchResult<any>>) => void;
    private option;
    constructor(option?: Partial<FetchOption>);
    use<T>(func: (fetcher: this, options?: T) => void, options?: T): void;
    setFetchOption(option: Partial<FetchOption>): void;
    getFetchOption(): FetchOption;
    private fetch;
    request: <Response_1, Request_1>(method: 'GET' | 'POST' | 'DELETE' | 'PUT' | 'HEAD' | 'PATCH', init: FetchInitType) => FetchMethod<Response_1, Request_1>;
    get: <Response_1 = any, Request_1 = any>(init: FetchInitType) => FetchMethod<Response_1, Request_1>;
    post: <Response_1 = any, Request_1 = any>(init: FetchInitType) => FetchMethod<Response_1, Request_1>;
    delete: <Response_1 = any, Request_1 = any>(init: FetchInitType) => FetchMethod<Response_1, Request_1>;
    put: <Response_1 = any, Request_1 = any>(init: FetchInitType) => FetchMethod<Response_1, Request_1>;
    head: <Response_1 = any, Request_1 = any>(init: FetchInitType) => FetchMethod<Response_1, Request_1>;
    patch: <Response_1 = any, Request_1 = any>(init: FetchInitType) => FetchMethod<Response_1, Request_1>;
}
export declare function createFetcher(option?: Partial<FetchOption>): Fetcher;
export declare const api: Fetcher;
